@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.83 2003/04/25 15:27:32 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.104 2003/11/07 18:47:30 richb Exp $
  *
  *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -46,6 +46,7 @@
 #define BUT_7     X->bas_buttons[0]        /* 7 */
 #define BUT_8     X->bas_buttons[1]        /* 8 */
 #define BUT_9     X->bas_buttons[2]        /* 9 */
+#define BUT_CLR   X->bas_buttons[7]        /* Clr */
 #define BUT_ACC   X->sci_buttons[7]        /* A */
 #define BUT_A     X->sci_buttons[24]       /* a */
 #define BUT_B     X->sci_buttons[25]       /* b */
@@ -53,14 +54,6 @@
 #define BUT_D     X->sci_buttons[16]       /* d */
 #define BUT_E     X->sci_buttons[17]       /* e */
 #define BUT_F     X->sci_buttons[18]       /* f */
-#ifdef WANTED
-#define BUT_ADD   X->bas_buttons[27]       /* + */
-#define BUT_SUB   X->bas_buttons[19]       /* - */
-#define BUT_MUL   X->bas_buttons[11]       /* x */
-#define BUT_DIV   X->bas_buttons[3]        /* / */
-#define BUT_PNT   X->bas_buttons[25]       /* . */
-#define BUT_EQ    X->bas_buttons[26]       /* = */
-#endif /*WANTED*/
 
 typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkAccelGroup *kbd_accel;
@@ -72,7 +65,9 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *aframe;                 /* ASCII window. */
     GtkWidget *aframe_ch;
     GtkWidget *base[MAXBASES];         /* Numeric base radio buttons. */
+    GtkWidget *con_dialog;             /* Edit constants dialog. */
     GtkWidget *disp[MAXDISPMODES];     /* Numeric display mode. */
+    GtkWidget *fun_dialog;             /* Edit functions dialog. */
     GtkWidget *hyp;                    /* Hyperbolic mode. */
     GtkWidget *inv;                    /* Inverse mode. */
     GtkWidget *kframe;                 /* Main window. */
@@ -87,11 +82,6 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *menus[MAXMENUS];
     GtkWidget *trig[MAXTRIGMODES];     /* Trigonometric mode. */
 
-    gboolean ctrl_l;             /* Set if the left Control key is pressed. */
-    gboolean ctrl_r;             /* Set if the right Control key is pressed. */
-    gboolean shft_l;             /* Set if the left Shift key is pressed. */
-    gboolean shft_r;             /* Set if the right Shift key is pressed. */
-
     GtkWidget *bas_buttons[BROWS * BCOLS];
     GtkWidget *fin_buttons[FROWS * FCOLS];
     GtkWidget *sci_buttons[SROWS * SCOLS];
@@ -100,11 +90,8 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *fin_frame;
     GtkWidget *sci_frame;
 
-    enum menu_type CFtype;
     Display *dpy;
 
-    GArray *cf_entries;
-
     int menuval;                  /* Index to button array at menu time. */
     struct button *mrec[MAXMENUS];
 } XObject;
@@ -139,7 +126,6 @@ static gboolean aframe_key_cb(GtkWidget *, GdkEventKey *, gpointer);
 static gboolean dismiss_aframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean dismiss_rframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean kframe_key_press_cb(GtkWidget *, GdkEventKey *, gpointer);
-static gboolean kframe_key_release_cb(GtkWidget *, GdkEventKey *, gpointer);
 
 static void about_cb(GtkWidget *, gpointer);
 static void add_cf_column(GtkTreeView *, gchar *, gint, gboolean);
@@ -195,7 +181,7 @@ static GtkItemFactoryEntry main_menu[] = {
 
     { N_("/_Help"),                NULL, NULL,    0,       "<Branch>" },
     { N_("/Help/_Contents"),       "F1", mb_proc, M_CONTENTS, "<StockItem>", GTK_STOCK_HELP },
-    { N_("/Help/_About"),    	   NULL, about_cb, 0, "<StockItem>", GNOME_STOCK_ABOUT },
+    { N_("/Help/_About"),    	   NULL, about_cb, M_ABOUT, "<StockItem>", GNOME_STOCK_ABOUT },
 };
 
 static GtkItemFactoryEntry acc_menu[] = {
@@ -223,12 +209,12 @@ static GtkItemFactoryEntry lshift_menu[] = {
     { N_("/7 places"),  NULL, menu_proc, '7', NULL },
     { N_("/8 places"),  NULL, menu_proc, '8', NULL },
     { N_("/9 places"),  NULL, menu_proc, '9', NULL },
-    { N_("/10 places"), NULL, menu_proc, 'a', NULL },
-    { N_("/11 places"), NULL, menu_proc, 'b', NULL },
-    { N_("/12 places"), NULL, menu_proc, 'c', NULL },
-    { N_("/13 places"), NULL, menu_proc, 'd', NULL },
-    { N_("/14 places"), NULL, menu_proc, 'e', NULL },
-    { N_("/15 places"), NULL, menu_proc, 'f', NULL },
+    { N_("/10 places"), NULL, menu_proc, 'A', NULL },
+    { N_("/11 places"), NULL, menu_proc, 'B', NULL },
+    { N_("/12 places"), NULL, menu_proc, 'C', NULL },
+    { N_("/13 places"), NULL, menu_proc, 'D', NULL },
+    { N_("/14 places"), NULL, menu_proc, 'E', NULL },
+    { N_("/15 places"), NULL, menu_proc, 'F', NULL },
 };
 
 static GtkItemFactoryEntry rshift_menu[] = {
@@ -241,12 +227,12 @@ static GtkItemFactoryEntry rshift_menu[] = {
     { N_("/7 places"),  NULL, menu_proc, '7', NULL },
     { N_("/8 places"),  NULL, menu_proc, '8', NULL },
     { N_("/9 places"),  NULL, menu_proc, '9', NULL },
-    { N_("/10 places"), NULL, menu_proc, 'a', NULL },
-    { N_("/11 places"), NULL, menu_proc, 'b', NULL },
-    { N_("/12 places"), NULL, menu_proc, 'c', NULL },
-    { N_("/13 places"), NULL, menu_proc, 'd', NULL },
-    { N_("/14 places"), NULL, menu_proc, 'e', NULL },
-    { N_("/15 places"), NULL, menu_proc, 'f', NULL },
+    { N_("/10 places"), NULL, menu_proc, 'A', NULL },
+    { N_("/11 places"), NULL, menu_proc, 'B', NULL },
+    { N_("/12 places"), NULL, menu_proc, 'C', NULL },
+    { N_("/13 places"), NULL, menu_proc, 'D', NULL },
+    { N_("/14 places"), NULL, menu_proc, 'E', NULL },
+    { N_("/15 places"), NULL, menu_proc, 'F', NULL },
 };
 
 
@@ -291,28 +277,32 @@ main(int argc, char **argv)
 static void
 about_cb(GtkWidget *widget, gpointer data)
 {
-    GtkWidget *about;        
+    static GtkWidget *about = NULL;
 
-    const gchar *authors[] = {
-        "Rich Burridge <rich.burridge@sun.com>",
-        NULL
-    };
-    const gchar *documenters[] = {
-        NULL
-    };
-    const gchar *translator_credits = _("translator_credits");
+    if (about == NULL) {
+        const gchar *authors[] = {
+            "Rich Burridge <rich.burridge@sun.com>",
+            NULL
+        };
+        const gchar *documenters[] = {
+            NULL
+        };
+        const gchar *translator_credits = _("translator_credits");
 
-    about = gnome_about_new(_("Gcalctool"), VERSION,
-                   "(C) 2003 the Free Software Foundation",
-                   _("Calculator with financial and scientific modes."),
-                   authors,
-                   documenters,
-                   strcmp(translator_credits, "translator_credits") != 0 ? 
-                          translator_credits : NULL,
-                   X->icon);
+        about = gnome_about_new(_("Gcalctool"), VERSION,
+                       "(C) 2003 the Free Software Foundation",
+                       _("Calculator with financial and scientific modes."),
+                       authors,
+                       documenters,
+                       strcmp(translator_credits, "translator_credits") != 0 ? 
+                              translator_credits : NULL,
+                       X->icon);
 
-    gtk_window_set_icon(GTK_WINDOW(about), X->icon);    
-    gtk_widget_show(about);
+        g_signal_connect(G_OBJECT(about), "destroy",
+                         G_CALLBACK(gtk_widget_destroyed), &about);
+        gtk_window_set_icon(GTK_WINDOW(about), X->icon);    
+    }
+    gtk_window_present(GTK_WINDOW(about));
 }
 
 
@@ -482,6 +472,7 @@ cell_edited(GtkCellRendererText *cell, const gchar *path_string,
 {
     GtkTreeModel *model = (GtkTreeModel *) data;
     GtkTreePath *path = gtk_tree_path_new_from_string(path_string);
+    GArray *entries = (GArray *) g_object_get_data(G_OBJECT(model), "entries");
     GtkTreeIter iter;
     gchar *old_text;
     gint *column, i;
@@ -496,10 +487,10 @@ cell_edited(GtkCellRendererText *cell, const gchar *path_string,
             g_free(old_text);
  
             i = gtk_tree_path_get_indices(path)[0];
-            g_free(g_array_index(X->cf_entries, CF_Item, i).value);
-            g_array_index(X->cf_entries, CF_Item, i).value = g_strdup(new_text);
+            g_free(g_array_index(entries, CF_Item, i).value);
+            g_array_index(entries, CF_Item, i).value = g_strdup(new_text);
             gtk_list_store_set(GTK_LIST_STORE(model), &iter, column,
-                           g_array_index(X->cf_entries, CF_Item, i).value, -1);
+                           g_array_index(entries, CF_Item, i).value, -1);
             break;
 
         case COLUMN_DESCRIPTION:
@@ -507,11 +498,10 @@ cell_edited(GtkCellRendererText *cell, const gchar *path_string,
             g_free(old_text);
  
             i = gtk_tree_path_get_indices(path)[0];
-            g_free(g_array_index(X->cf_entries, CF_Item, i).description);
-            g_array_index(X->cf_entries, CF_Item, i).description = 
-                                                             g_strdup(new_text);
+            g_free(g_array_index(entries, CF_Item, i).description);
+            g_array_index(entries, CF_Item, i).description = g_strdup(new_text);
             gtk_list_store_set(GTK_LIST_STORE(model), &iter, column,
-                     g_array_index(X->cf_entries, CF_Item, i).description, -1);
+                     g_array_index(entries, CF_Item, i).description, -1);
             break;
     }
  
@@ -524,6 +514,9 @@ cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
 {
     CF_Item item;
     int i;
+    enum menu_type mtype = (enum menu_type)
+                               g_object_get_data(G_OBJECT(dialog), "mtype");
+    GArray *entries = (GArray *) g_object_get_data(G_OBJECT(dialog), "entries");
 
     if (id == GTK_RESPONSE_HELP) {
         GError *error = NULL;
@@ -548,9 +541,9 @@ cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
 
     if (id == GTK_RESPONSE_ACCEPT) {
         for (i = 0; i < MAXCONFUN; i++) {
-            item = g_array_index(X->cf_entries, CF_Item, i);
+           item = g_array_index(entries, CF_Item, i);
 
-            if (X->CFtype == M_CON) {
+           if (mtype == M_CON) {
                 MPstr_to_num(item.value, DEC, v->MPcon_vals[i]);
                 STRCPY(v->con_names[i], item.description);
                 put_constant(i, item.value, item.description);
@@ -561,17 +554,22 @@ cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
             }
         }    
 
-        create_con_fun_menu(X->CFtype);
+        create_con_fun_menu(mtype);
     }
 
     gtk_widget_destroy(GTK_WIDGET(dialog));
+    if (mtype == M_CON) {
+	X->con_dialog = NULL;
+    } else {
+	X->fun_dialog = NULL;
+    }
 }
 
 
 static void
 create_aframe()  /* Create auxiliary frame for ASC key. */
 {
-    GtkWidget *vbox, *hbox, *button_hbox, *label, *separator;
+    GtkWidget *vbox, *hbox, *button_hbox, *label;
     GtkWidget *insert_button, *cancel_button;
 
     X->aframe = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -581,18 +579,18 @@ create_aframe()  /* Create auxiliary frame for ASC key. */
     gtk_window_set_type_hint(GTK_WINDOW(X->aframe), 
                              GDK_WINDOW_TYPE_HINT_DIALOG);
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_vbox_new(FALSE, 12);
     gtk_widget_show(vbox);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), GNOME_PAD_SMALL);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
     gtk_container_add(GTK_CONTAINER(X->aframe), vbox);
 
-    hbox = gtk_hbox_new(FALSE, 0);
+    hbox = gtk_hbox_new(FALSE, 6);
     gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, GNOME_PAD);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
     label = gtk_label_new_with_mnemonic(_("Ch_aracter:"));
     gtk_widget_show(label);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 
     X->aframe_ch = gtk_entry_new();
@@ -600,21 +598,17 @@ create_aframe()  /* Create auxiliary frame for ASC key. */
     gtk_entry_set_max_length(GTK_ENTRY(X->aframe_ch), 1);
     gtk_widget_show(X->aframe_ch);
     gtk_box_pack_start(GTK_BOX(hbox), X->aframe_ch, 
-                       FALSE, FALSE, GNOME_PAD_SMALL);
-
-    separator = gtk_hseparator_new();
-    gtk_widget_show(separator);
-    gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
+                       FALSE, FALSE, 0);
 
     button_hbox = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(button_hbox), GTK_BUTTONBOX_END);
     gtk_widget_ref(button_hbox);
     gtk_widget_show(button_hbox);
     gtk_box_pack_start(GTK_BOX(vbox), button_hbox, TRUE, TRUE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(button_hbox), GNOME_PAD_SMALL);
-    gtk_box_set_spacing(GTK_BOX(button_hbox), GNOME_PAD);
+    gtk_box_set_spacing(GTK_BOX(button_hbox), 12);
 
     cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
     gtk_widget_ref(cancel_button);
     gtk_widget_show(cancel_button);
     gtk_box_pack_start(GTK_BOX(button_hbox), cancel_button, FALSE, FALSE, 0);
@@ -643,125 +637,144 @@ create_aframe()  /* Create auxiliary frame for ASC key. */
 
 
 static GtkTreeModel *
-create_cf_model(enum menu_type mtype)
+create_cf_model(enum menu_type mtype, GtkWidget *dialog)
 {
     gint i = 0;
     CF_Item n;
     GtkListStore *model;
     GtkTreeIter iter;
- 
-    X->cf_entries = g_array_sized_new(FALSE, FALSE, sizeof(CF_Item), 1);
+    GArray *entries = g_array_sized_new(FALSE, FALSE, sizeof(CF_Item), 1);
  
     for (i = 0; i < MAXCONFUN; i++) {
         n.number = i;
         if (mtype == M_CON) {
-            n.value       = g_strdup(make_number(v->MPcon_vals[i], FALSE));
+            n.value = g_strdup(make_number(v->MPcon_vals[i], DEC, FALSE, TRUE));
             n.description = g_strdup(v->con_names[i]);
         } else {
             n.value       = g_strdup(v->fun_vals[i]);
             n.description = g_strdup(v->fun_names[i]);
         }
         n.editable = TRUE;
-        g_array_append_vals(X->cf_entries, &n, 1);
+        g_array_append_vals(entries, &n, 1);
     }
  
     model = gtk_list_store_new(NUM_COLUMNS, G_TYPE_INT, G_TYPE_STRING,
                                G_TYPE_STRING, G_TYPE_BOOLEAN);
  
-    for (i = 0; i < X->cf_entries->len; i++) {
+    for (i = 0; i < entries->len; i++) {
         gtk_list_store_append(model, &iter);
    
         gtk_list_store_set(model, &iter,
                            COLUMN_NUMBER,
-                           g_array_index(X->cf_entries, CF_Item, i).number,
+                           g_array_index(entries, CF_Item, i).number,
                            COLUMN_VALUE,
-                           g_array_index(X->cf_entries, CF_Item, i).value,
+                           g_array_index(entries, CF_Item, i).value,
                            COLUMN_DESCRIPTION,
-                           g_array_index(X->cf_entries, CF_Item, i).description,
+                           g_array_index(entries, CF_Item, i).description,
                            COLUMN_EDITABLE,
-                           g_array_index(X->cf_entries, CF_Item, i).editable,
+                           g_array_index(entries, CF_Item, i).editable,
                            -1);
     }
-   
+    g_object_set_data(G_OBJECT(model), "entries", (gpointer) entries);
+    g_object_set_data(G_OBJECT(dialog), "entries", (gpointer) entries);
+
     return(GTK_TREE_MODEL(model));
 }
 
 
 /* Create popup window for editing constants/functions. */
 
-static void
-create_cfframe(enum menu_type mtype)
+static GtkWidget *
+create_cfframe(enum menu_type mtype, GtkWidget *dialog)
 {
     const gchar *title;
     GdkGeometry geometry;
     GtkTreeModel *model;
-    GtkWidget *dialog, *label, *sw, *treeview, *vbox;
+    GtkWidget *label, *sw, *treeview, *vbox;
+    gchar *str;
 
     title = (mtype == M_CON) ? _("Edit Constants") : _("Edit Functions");
-    geometry.min_width = 360;
+    geometry.min_width = 380;
     geometry.min_height = 300;
-    dialog = gtk_dialog_new_with_buttons(title, NULL,
-                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_STOCK_HELP,   GTK_RESPONSE_HELP,
-                                         GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-                                         GTK_STOCK_OK,     GTK_RESPONSE_ACCEPT,
-                                         NULL);
+    if (dialog == NULL) {
+        dialog = gtk_dialog_new_with_buttons(title, NULL,
+                          GTK_DIALOG_DESTROY_WITH_PARENT,
+                          GTK_STOCK_HELP,   GTK_RESPONSE_HELP,
+                          GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+                          GTK_STOCK_OK,     GTK_RESPONSE_ACCEPT,
+                          NULL);
 
-    gtk_dialog_set_default_response(GTK_DIALOG(dialog), 
-                                    GTK_RESPONSE_ACCEPT);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
-    gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
-    gtk_window_set_transient_for(GTK_WINDOW(dialog), 
-                                 GTK_WINDOW(X->kframe));
-    gtk_window_set_geometry_hints(GTK_WINDOW(dialog), dialog,
-                                  &geometry, GDK_HINT_MIN_SIZE);
-    vbox = gtk_vbox_new(FALSE, 6);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
+        g_object_set_data(G_OBJECT(dialog), "mtype", (gpointer) mtype);
+        gtk_dialog_set_default_response(GTK_DIALOG(dialog), 
+                                        GTK_RESPONSE_ACCEPT);
+        gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
+        gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
+        gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
+        gtk_window_set_transient_for(GTK_WINDOW(dialog), 
+                                     GTK_WINDOW(X->kframe));
+        gtk_window_set_geometry_hints(GTK_WINDOW(dialog), dialog,
+                                      &geometry, GDK_HINT_MIN_SIZE);
+        vbox = gtk_vbox_new(FALSE, 6);
+        gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
-    label = gtk_label_new(_("Click a value or description to edit it:"));
-    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+	str = g_strconcat("<small><i><b>", _("Note:"), "</b> ", 
+            _("All constant values are specified in the decimal numeric base."),
+            "</i></small>", NULL); 
+	label = gtk_label_new(_(str));
+	g_free(str);
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+	gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
-    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+        label = gtk_label_new_with_mnemonic(
+                    _("Click a _value or description to edit it:"));
+        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+        gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
-    sw = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-                                        GTK_SHADOW_ETCHED_IN);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-                                   GTK_POLICY_AUTOMATIC,
-                                   GTK_POLICY_NEVER);
-    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
+        sw = gtk_scrolled_window_new(NULL, NULL);
+        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+                                            GTK_SHADOW_NONE);
+        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+                                       GTK_POLICY_AUTOMATIC,
+                                       GTK_POLICY_NEVER);
+        gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
 
-    model = create_cf_model(mtype);
+	model = create_cf_model(mtype, dialog);
 
-    treeview = gtk_tree_view_new_with_model(model);
-    g_object_unref(model);
-    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
-    gtk_tree_selection_set_mode(
-                         gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)),
-                                                     GTK_SELECTION_SINGLE);
+        treeview = gtk_tree_view_new_with_model(model);
+        gtk_label_set_mnemonic_widget(GTK_LABEL(label), treeview);
+        g_object_unref(model);
+        gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
+        gtk_tree_selection_set_mode(
+                gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)),
+                                            GTK_SELECTION_SINGLE);
 
-    add_cf_column(GTK_TREE_VIEW(treeview), _("No."),
-                  COLUMN_NUMBER, FALSE);
-    add_cf_column(GTK_TREE_VIEW(treeview), _("Value"),
-                  COLUMN_VALUE, TRUE);
-    add_cf_column(GTK_TREE_VIEW(treeview), _("Description"),
-                  COLUMN_DESCRIPTION, TRUE);
+        add_cf_column(GTK_TREE_VIEW(treeview), _("No."),
+                      COLUMN_NUMBER, FALSE);
+        add_cf_column(GTK_TREE_VIEW(treeview), _("Value"),
+                      COLUMN_VALUE, TRUE);
+        add_cf_column(GTK_TREE_VIEW(treeview), _("Description"),
+                      COLUMN_DESCRIPTION, TRUE);
 
-    gtk_container_add(GTK_CONTAINER(sw), treeview);
+        gtk_container_add(GTK_CONTAINER(sw), treeview);
 
-    g_signal_connect(G_OBJECT(dialog), "response",
-                     G_CALLBACK(cfframe_response_cb), (gpointer) model);
+        g_signal_connect(G_OBJECT(dialog), "response",
+                         G_CALLBACK(cfframe_response_cb), (gpointer) model);
 
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
+        gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
+    }
     gtk_widget_show_all(dialog);
+
+    return(dialog);
 }
 
 
 static char *
-item_factory_translate_func (const char *path, gpointer func_data)
+item_factory_translate_func(const char *path, gpointer func_data)
 {
-    return _(path);
+    return(_(path));
 }
 
 
@@ -779,6 +792,7 @@ create_kframe()
         v->tool_label = malloc(strlen(_("Calculator")) + strlen(hn) + 3);
 
         SPRINTF(v->tool_label, "%s %s", _("Calculator"), hn);
+        g_free(hn);
     } else {
         read_str(&v->tool_label, v->titleline);
     }
@@ -819,9 +833,9 @@ create_kframe()
     gtk_text_buffer_create_tag(buffer, "x-large", "scale", PANGO_SCALE_X_LARGE, 
                                NULL);			       
 
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(X->display_item), FALSE);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(X->display_item), TRUE);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(X->display_item), FALSE);
-    GTK_WIDGET_UNSET_FLAGS(X->display_item, GTK_CAN_FOCUS);
+    GTK_WIDGET_SET_FLAGS(X->display_item, GTK_CAN_FOCUS);
 
     gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(X->display_item), 12);
     gtk_text_view_set_pixels_below_lines(GTK_TEXT_VIEW(X->display_item), 12);
@@ -853,11 +867,10 @@ create_kframe()
     gtk_window_add_accel_group(GTK_WINDOW(X->kframe), X->kbd_accel);
     grey_buttons(v->base);
     gtk_window_set_icon(GTK_WINDOW(X->kframe), X->icon);
+    gtk_window_set_focus(GTK_WINDOW(X->kframe), GTK_WIDGET(BUT_CLR));
 
     g_signal_connect(G_OBJECT(X->kframe), "key_press_event",
                      G_CALLBACK(kframe_key_press_cb), NULL);
-    g_signal_connect(G_OBJECT(X->kframe), "key_release_event",
-                     G_CALLBACK(kframe_key_release_cb), NULL);
 
     switch (v->modetype) {
         case FINANCIAL:
@@ -887,7 +900,7 @@ create_mem_menu(enum menu_type mtype)
         SPRINTF(mstr, "<span weight=\"bold\">%s%d:</span>    %s",
 	/* translators: R is the short form of register used inter alia
 	in popup menus */
-                _("R"), i, make_number(v->MPmvals[i], FALSE));
+                _("R"), i, make_number(v->MPmvals[i], v->base, FALSE, TRUE));
         create_menu_item_with_markup(mstr, m, i);
     }
 }
@@ -897,6 +910,7 @@ static GtkWidget *
 create_mode_panel(GtkWidget *main_vbox)
 {
     int i;
+    AtkObject *access_object;
     GtkWidget *base_hbox, *disp_hbox, *trig_hbox;
     GtkWidget *row1_hbox, *row2_hbox, *frame, *vbox;
     GSList *base_gr = NULL;
@@ -927,6 +941,8 @@ create_mode_panel(GtkWidget *main_vbox)
         gtk_box_pack_start(GTK_BOX(trig_hbox), X->trig[i], FALSE, FALSE, 0);
         gtk_radio_button_set_group(GTK_RADIO_BUTTON(X->trig[i]), trig_gr);
         trig_gr = gtk_radio_button_get_group(GTK_RADIO_BUTTON(X->trig[i]));
+        access_object = gtk_widget_get_accessible(X->trig[i]);
+        atk_object_set_name(access_object, _(ttype_desc[i]));
         g_signal_connect(G_OBJECT(X->trig[i]), "toggled",
                          G_CALLBACK(trig_cb), NULL);
     }
@@ -946,6 +962,8 @@ create_mode_panel(GtkWidget *main_vbox)
         gtk_box_pack_start(GTK_BOX(base_hbox), X->base[i], FALSE, FALSE, 0);
         gtk_radio_button_set_group(GTK_RADIO_BUTTON(X->base[i]), base_gr);
         base_gr = gtk_radio_button_get_group(GTK_RADIO_BUTTON(X->base[i]));
+        access_object = gtk_widget_get_accessible(X->base[i]);
+        atk_object_set_name(access_object, _(base_desc[i]));
         g_signal_connect(G_OBJECT(X->base[i]), "toggled",
                          G_CALLBACK(base_cb), NULL);
     }
@@ -955,20 +973,20 @@ create_mode_panel(GtkWidget *main_vbox)
 /* Make Hyp and Inv trigonometric check boxes. */
 
     X->inv = gtk_check_button_new_with_mnemonic(_("_Inv"));
-    gtk_tooltips_set_tip(X->tips, X->inv, 
-                         _("Set inverse option for trigonometric functions"), "");
+    gtk_tooltips_set_tip(X->tips, X->inv, _(inv_desc), "");
     gtk_widget_show(X->inv);
     gtk_box_pack_start(GTK_BOX(row2_hbox), X->inv, FALSE, FALSE, 0);
-    g_signal_connect(G_OBJECT(X->inv), "toggled",
-                      G_CALLBACK(inv_cb), NULL);
+    access_object = gtk_widget_get_accessible(X->inv);
+    atk_object_set_name(access_object, _(inv_desc));
+    g_signal_connect(G_OBJECT(X->inv), "toggled", G_CALLBACK(inv_cb), NULL);
 
     X->hyp = gtk_check_button_new_with_mnemonic(_("H_yp"));
-    gtk_tooltips_set_tip(X->tips, X->hyp, 
-                         _("Set hyperbolic option for trigonometric functions"), "");
+    gtk_tooltips_set_tip(X->tips, X->hyp, _(hyp_desc), "");
     gtk_widget_show(X->hyp);
     gtk_box_pack_start(GTK_BOX(row2_hbox), X->hyp, FALSE, FALSE, 0);
-    g_signal_connect(G_OBJECT(X->hyp), "toggled",
-                      G_CALLBACK(hyp_cb), NULL);
+    access_object = gtk_widget_get_accessible(X->hyp);
+    atk_object_set_name(access_object, _(hyp_desc));
+    g_signal_connect(G_OBJECT(X->hyp), "toggled", G_CALLBACK(hyp_cb), NULL);
 
 /* Make display type radio button widgets. */
 
@@ -983,6 +1001,8 @@ create_mode_panel(GtkWidget *main_vbox)
         gtk_box_pack_start(GTK_BOX(disp_hbox), X->disp[i], FALSE, FALSE, 0);
         gtk_radio_button_set_group(GTK_RADIO_BUTTON(X->disp[i]), disp_gr);
         disp_gr = gtk_radio_button_get_group(GTK_RADIO_BUTTON(X->disp[i]));
+        access_object = gtk_widget_get_accessible(X->disp[i]);
+        atk_object_set_name(access_object, _(dtype_desc[i]));
         g_signal_connect(G_OBJECT(X->disp[i]), "toggled",
                          G_CALLBACK(disp_cb), NULL);
     }
@@ -1017,7 +1037,7 @@ create_rframe()
 
     for (i = 0; i < MAXREGS; i++) {
         SPRINTF(line, "<span weight=\"bold\">%s%1d:</span>   %s", 
-                _("R"), i,  make_number(v->MPmvals[i], FALSE));
+                _("R"), i,  make_number(v->MPmvals[i], v->base, FALSE, TRUE));
         X->regs[i] = gtk_label_new("");
         gtk_label_set_markup(GTK_LABEL(X->regs[i]), line);
         SPRINTF(name, "register_label%1d", i);
@@ -1078,7 +1098,8 @@ create_con_fun_menu(enum menu_type mtype)
         invalid = 0;
         if (mtype == M_CON) {
             SPRINTF(mline, "<span weight=\"bold\">%s%1d:</span> %s [%s]", 
-                    _("C"), i, make_number(v->MPcon_vals[i], FALSE), 
+                    _("C"), i, 
+                    make_number(v->MPcon_vals[i], DEC, FALSE, TRUE), 
                     v->con_names[i]);
         } else {
             if (!strlen(v->fun_vals[i])) {
@@ -1165,6 +1186,7 @@ get_display()              /* The Copy function key has been pressed. */
         free(v->shelf);
     }
     v->shelf = g_locale_from_utf8(string, strlen(string), NULL, NULL, NULL);
+    g_free(string);
 
     gtk_clipboard_set_text(gtk_clipboard_get(X->clipboard_atom), v->shelf, -1);
 }
@@ -1293,26 +1315,19 @@ inv_cb(GtkToggleButton *button, gpointer user_data)
 
 
 static int
-check_vals(int n, int keyval, 
+check_vals(int n, int keyval, int state,
            struct button buttons[], GtkWidget *gtk_buttons[])
 {
     int i, j;
 
+    state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK);
     for (i = 0; i < n; i++) {
         j = 0;
         while (buttons[i].value[j] != 0) {
             if (buttons[i].value[j] == keyval) {
-                if ((buttons[i].mods[j] == 0) &&
-                    ((X->ctrl_l | X->ctrl_r | X->shft_l | X->shft_r) == 0)) {
-                    button_proc(GTK_BUTTON(gtk_buttons[i]), NULL);
-                    return(TRUE);
-                } else if ((buttons[i].mods[j] == GDK_SHIFT_MASK) &&
-                           ((X->shft_l | X->shft_r) == 1)) {
-                    button_proc(GTK_BUTTON(gtk_buttons[i]), NULL);
-                    return(TRUE);
-                } else if ((buttons[i].mods[j] == GDK_CONTROL_MASK) &&
-                           ((X->ctrl_l | X->ctrl_r) == 1)) {
-                    button_proc(GTK_BUTTON(gtk_buttons[i]), NULL);
+                if (buttons[i].mods[j] == state) {
+                    g_signal_emit_by_name(GTK_WIDGET(gtk_buttons[i]), 
+                                          "activate");
                     return(TRUE);
                 }
             }
@@ -1327,42 +1342,29 @@ check_vals(int n, int keyval,
 static gboolean
 kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
+    GtkWidget *focus;
     int retval = FALSE;
 
-    if (event->keyval == GDK_Control_L) {
-        X->ctrl_l = TRUE;
-        return(TRUE);
-    } else if (event->keyval == GDK_Control_R) {
-        X->ctrl_r = TRUE;
-        return(TRUE);
-    } else if (event->keyval == GDK_Shift_L) {
-        X->shft_l = TRUE;
-        return(TRUE);
-    } else if (event->keyval == GDK_Shift_R) {
-        X->shft_r = TRUE;
-	return(TRUE);
-    }
-
     switch (v->modetype) {
         case BASIC:
-            retval = check_vals(B_NOBUTTONS, event->keyval, 
+            retval = check_vals(B_NOBUTTONS, event->keyval, event->state,
                                 b_buttons, X->bas_buttons);
             break;
 
         case FINANCIAL:
-            retval = check_vals(B_NOBUTTONS, event->keyval,
+            retval = check_vals(B_NOBUTTONS, event->keyval, event->state,
             			b_buttons, X->bas_buttons);
             if (retval != TRUE) {
-                retval = check_vals(F_NOBUTTONS, event->keyval, 
+                retval = check_vals(F_NOBUTTONS, event->keyval, event->state,
                                     f_buttons, X->fin_buttons);
             }
             break;
 
         case SCIENTIFIC:
-            retval = check_vals(B_NOBUTTONS, event->keyval,
+            retval = check_vals(B_NOBUTTONS, event->keyval, event->state,
                                 b_buttons, X->bas_buttons);
             if (retval != TRUE) {
-                retval = check_vals(S_NOBUTTONS, event->keyval, 
+                retval = check_vals(S_NOBUTTONS, event->keyval, event->state,
                                     s_buttons, X->sci_buttons);
             }
             break;
@@ -1372,27 +1374,6 @@ kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 }
 
 
-static gboolean
-kframe_key_release_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
-{
-    if (event->keyval == GDK_Control_L) {
-        X->ctrl_l = FALSE;
-        return(TRUE);
-    } else if (event->keyval == GDK_Control_R) {
-        X->ctrl_r = FALSE;
-        return(TRUE);
-    } else if (event->keyval == GDK_Shift_L) {
-        X->shft_l = FALSE;
-        return(TRUE);
-    } else if (event->keyval == GDK_Shift_R) {
-        X->shft_r = FALSE;
-        return(TRUE);
-    }
-
-    return(FALSE);
-}
-
-
 void
 load_resources()        /* Load gconf configuration database for gcalctool. */
 { 
@@ -1457,6 +1438,16 @@ make_hostname(Display *dpy)
 }
 
 
+static void
+set_accessible_name(GtkWidget *widget, struct button button)
+{
+    AtkObject *access_object = gtk_widget_get_accessible(widget);
+
+    atk_object_set_name(access_object,
+                        (button.astr == NULL) ? button.hstr : button.astr);
+}
+
+
 static GtkWidget *
 make_but_frame(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                struct button buttons[], int rows, int cols, char *tag)
@@ -1488,12 +1479,15 @@ make_but_frame(GtkWidget *vbox, GtkWidget **Gtk_buttons,
             } else {
                 Gtk_buttons[n] = make_menu_button(label, j*cols + i);
             }
+            set_accessible_name(Gtk_buttons[n], buttons[n]);
             g_signal_connect(G_OBJECT(Gtk_buttons[n]), "clicked",
                             G_CALLBACK(button_proc), (gpointer) (j*cols + i));
             SPRINTF(name, "%s_button%1d", tag, n);
             gtk_widget_set_name(Gtk_buttons[n], name);
-            gtk_tooltips_set_tip(X->tips, Gtk_buttons[n],
-                                 _(buttons[n].hstr), "");
+            if (buttons[n].hstr != NULL) {
+                gtk_tooltips_set_tip(X->tips, Gtk_buttons[n],
+                                     _(buttons[n].hstr), "");
+            }
             g_object_set_data(G_OBJECT(Gtk_buttons[n]),
                               "button", &buttons[n]);
             gtk_widget_ref(Gtk_buttons[n]);
@@ -1625,6 +1619,7 @@ menu_button_key_press_cb(GtkWidget *widget,
         loc.y += widget->allocation.y;
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL, menu_pos_func,
                        (gpointer) &loc, event->keyval, event->time);
+        return(TRUE);
     }
 
     return(FALSE);
@@ -1748,8 +1743,13 @@ mstz_proc(gpointer data, int choice, GtkWidget *item)
 static void
 new_cf_value(GtkMenuItem *item, gpointer user_data)
 {
-    X->CFtype = (enum menu_type) user_data;
-    create_cfframe(X->CFtype);
+    enum menu_type mtype = (enum menu_type) user_data;
+
+    if (mtype == M_CON) {
+        X->con_dialog = create_cfframe(mtype, X->con_dialog);
+    } else {
+        X->fun_dialog = create_cfframe(mtype, X->fun_dialog);
+    }
 }
 
 
@@ -1842,22 +1842,72 @@ set_display(char *str)
 {
     GtkTextBuffer *buffer;
     GtkTextIter start, end;
-    gchar *utf8_str;
 
-    utf8_str = g_locale_to_utf8(str, strlen(str), NULL, NULL, NULL);
-	
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X->display_item));
-    gtk_text_buffer_get_bounds (buffer, &start, &end);
+    gtk_text_buffer_get_bounds(buffer, &start, &end);
     gtk_text_buffer_delete(buffer, &start, &end);
 
     gtk_text_buffer_insert_with_tags_by_name(buffer,
                                              &end,
-                                             utf8_str != NULL && strlen(utf8_str) != 0 ? utf8_str : " ",
+                                             str != NULL && strlen(str) != 0 ? str : " ",
                                              -1,
                                              "x-large",
                                              NULL);
+}
 
-    g_free(utf8_str);
+
+#define SET_MENUBAR_ITEM_STATE(i, state) \
+          gtk_widget_set_sensitive( \
+          gtk_item_factory_get_widget_by_action(X->mb_fact, i), state)
+
+
+/* When an error condition occurs:
+ *
+ * - make insensitive all buttons except Clr.
+ * - make all Scientific mode toggles and checkboxes insensitive.
+ * - make all menubar items insensitive except:
+ *     Calculator->Quit
+ *     Help->Contents
+ *
+ * When the error condition is cleared, resensitise everything, setting
+ * the numeric base buttons correctly.
+ */
+
+void
+set_error_state(int error)
+{
+    int i;
+
+    v->error = error;
+
+    for (i = 0; i < (BROWS * BCOLS); i++) {
+        set_button_state(X->bas_buttons[i], !v->error);
+    }
+    set_button_state(BUT_CLR, TRUE);    /* Clr button always sensitive. */
+
+    for (i = 0; i < (FROWS * FCOLS); i++) {
+        set_button_state(X->fin_buttons[i], !v->error);
+    }
+
+    for (i = 0; i < (SROWS * SCOLS); i++) {
+        set_button_state(X->sci_buttons[i], !v->error);
+    }
+
+    if (!v->error) {
+        grey_buttons(v->base);
+    }
+
+    gtk_widget_set_sensitive(X->mode_panel, !v->error);
+
+    SET_MENUBAR_ITEM_STATE(M_COPY,   !v->error);
+    SET_MENUBAR_ITEM_STATE(M_PASTE,  !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_ASCII,  !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_BASIC,  !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_FIN,    !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_SCI,    !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_ZEROES, !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_REGS,   !v->error); 
+    SET_MENUBAR_ITEM_STATE(M_ABOUT,  !v->error);
 }
 
 
